/* Copyright (C) 2025-2025 Warsaw University of Technology, Warsaw
    SPDX-License-Identifier: GPL-3.0-only
    Authors: Daniel Wielanek [committer] */
#ifndef CBMROOT_V0_ANALYSIS_PWGC2F_FEMTOSCOPY_HAL_HELPERS_HALCBMV0BUILDER_H_
#define CBMROOT_V0_ANALYSIS_PWGC2F_FEMTOSCOPY_HAL_HELPERS_HALCBMV0BUILDER_H_

#include "CbmAnaTreeContainer.h"

#include <Hal/Task.h>

class CbmAnaTreeV0Container;
class TClonesArray;

namespace AnalysisTree
{
  class Particle;
}


class HalCbmV0Builder : public Hal::Task {
  struct v0rawInfo {
    float chi2geo, chi2_prim_first;
    float chi2_prim_second, chi2_topo;
    float cosine_first, cosine_second, cosine_topo;
    float distance, l, l_dl;
    float mass, px, py, pz;
    float x, y, z;
    int dau1id, dau2id, pid, id;
    int mcid;
#ifdef DEBUG_EXTR
    int dau1px, dau1py, dau1pz, dau1x, dau1y, dau1z;
    int dau2px, dau2py, dau2pz, dau2x, dau2y, dau2z;
#endif
  };
  Int_t fPidMom;
  Int_t fPidPosDau;
  Int_t fPidNegDau;
  Bool_t fBeFriendly = {kFALSE};
  Double_t fMass     = {0};
  AnaTreeV0Ids fFields;
  CbmAnaTreeV0Container* fV0Container     = {nullptr};
  CbmAnaTreeRecoContainer* fRecoContainer = {nullptr};
  CbmAnaTreeMcContainer* fSimContainer    = {nullptr};
  TClonesArray* fV0Temp                   = {nullptr};
  Bool_t fOwner                           = {kFALSE};
  v0rawInfo Convert(const AnalysisTree::Particle& p);

 public:
  HalCbmV0Builder(Int_t motherPid = 3321, Int_t posdau = 2212, Int_t negdau = -211);
  virtual EInitFlag Init();
  void BeFriendly() { fBeFriendly = kTRUE; }
  virtual void Exec(Option_t* option = "");
  virtual ~HalCbmV0Builder();
  ClassDef(HalCbmV0Builder, 1)
};

#endif /* CBMROOT_V0_ANALYSIS_PWGC2F_FEMTOSCOPY_HAL_HELPERS_HALCBMV0BUILDER_H_ */
