/* Copyright (C) 2025-2025 Warsaw University of Technology, Warsaw
    SPDX-License-Identifier: GPL-3.0-only
    Authors: Daniel Wielanek [committer] */
#include "RootManager.h"

#include <FairRootManager.h>

#include <RtypesCore.h>
#include <TCollection.h>
#include <TFile.h>
#include <TList.h>
#include <TObjString.h>

#include <Hal/InputDataInfo.h>

namespace Hal
{
  namespace Fair
  {
    RootManager::RootManager()
      : Hal::IOManager(new Hal::InputDataInfo(FairRootManager::Instance()->GetInFile()->GetName()))
    {
      fFairManager = FairRootManager::Instance();
      if (fFairManager->GetInFile()) {
        fDataInfo->OverwriteSourceName(fFairManager->GetInFile()->GetName());
      }
    }

    TObject* RootManager::GetObject(const char* BrName) { return fFairManager->GetObject(BrName); }

    TFile* RootManager::GetInFile() { return fFairManager->GetInFile(); }

    void RootManager::UpdateBranches() { fFairManager->UpdateBranches(); }

    void RootManager::SetInChain(TChain* tempChain, Int_t ident) { fFairManager->SetInChain(tempChain, ident); }

    void RootManager::RegisterInternal(const char* name, const char* folderName, TNamed* obj, Bool_t toFile)
    {
      fFairManager->Register(name, folderName, obj, toFile);
    }

    void RootManager::RegisterInternal(const char* name, const char* Foldername, TCollection* obj, Bool_t toFile)
    {
      fFairManager->Register(name, Foldername, obj, toFile);
    }

    void RootManager::RefreshBranchList()
    {
      fFairManager = FairRootManager::Instance();
      TList* l     = fFairManager->GetBranchNameList();
      for (int i = 0; i < l->GetEntries(); i++) {
        TString name = ((TObjString*) l->At(i))->GetString();
        Int_t stat   = fFairManager->CheckBranch(name);
        TObject* obj = fFairManager->GetObject(name);
        switch (stat) {
          case 0:  // not exist
            break;
          case 1:  // in file
            AddBranch(name, obj, BranchInfo::EFlag::kInActive);
            break;
          case 2:  // memory
            AddBranch(name, obj, BranchInfo::EFlag::kVirtual);
            break;
        }
      }
    }

    Bool_t RootManager::InitInternal()
    {
      if (fFairManager == nullptr) {
        fFairManager = FairRootManager::Instance();
        if (fFairManager->GetInFile()) {
          fDataInfo->OverwriteSourceName(fFairManager->GetInFile()->GetName());
        }
      }
      RefreshBranchList();
      return kTRUE;
    }

    void RootManager::CdToOutput()
    {
      if (!fFairManager) return;
      auto file = fFairManager->GetOutFile();
      if (file) file->cd();
    }

  }  // namespace Fair
}  // namespace Hal
